/*

Read a symbol table file and create two AVL binary search trees from it.
One tree is sorted by symbol name, the other by symbol value.
Print the resulting trees in the format defined by the dot program.
Input from stdin or named file, output to stdout.

Doug Johnson, djohnson@cs.washington.edu, April 2002 for CSE 373

The output from this program is input to dot.  You can run the two
programs one after the other like this in Linux.

  ./mainTree ../symbols/tvsymbols.txt > tree.dot
  dot -Tps -o tree.ps tree.dot

There is a command file winrunit.bat that does the equivalent commands
in Windows.

  Debug\tree.exe ..\symbols\tvsymbols.txt > wintree.dot
  dot -Tps -o wintree.ps wintree.dot

You can also pipe the two programs together and avoid the intermediate
file.

  ./mainTree ../symbols/tvsymbols.txt | dot -Tps -o tree.ps

or 

  Debug\tree.exe ..\symbols\tvsymbols.txt | dot -Tps -o wintree.ps

The dot program is part of the graphics visualization tool set that
is available from ATT.  <http://www.research.att.com/sw/tools/graphviz/>

The format of each row of the input symbol table file is simply

 "<numeric value> <symbol name>"

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "reporter.h"
#include "symbol.h"
#include "tree.h"
#include "plottree.h"

#define MAX_NAME_LENGTH 100

void symbolPrintLabel(void *E);

int main(int argc,char *argv[]) {

  FILE *src;			/* input file */
  long count;			/* number of Symbols defined */
  int idx;			/* loop control */
  
  SearchTree Ta;		/* root of a tree sorted by name*/
  SearchTree Tb;		/* root of a tree sorted by value */

  struct Symbol sym;		/* a Symbol struct */
  struct Symbol *symP;		/* pointer to a Symbol struct */
  struct Symbol **symTable;	/* array of pointers to Symbol structs */

  Position node;		/* a node in the tree */

  char *findNames[] = { 
    "_start","getTreeAuthor","freeSymbols",NULL };

  /* read from stdin or the named file */

  if (argc == 1) {
    src = stdin;
  } else if (argc == 2) {
    src = fopen(argv[1],"r");
    if (src == NULL) {
      printf("main: file open error on %s.\n",argv[1]);
      return 1;
    }
  } else {  
    printf("usage: %s [<filename>]\n",argv[0]);
    return 1;
  }

  /* Read the symbol definitions */

  symTable = readSymbols(src);
  fclose(src);

  /* Build the trees, one symbol at a time */

  Ta = CreateTree();
  Tb = CreateTree();

  count = 0;
  while (symTable[count] != NULL) {
    Ta = InsertTreeNode(Ta,symbolCompareByName,symTable[count]);
    Tb = InsertTreeNode(Tb,symbolCompareByValue,symTable[count]);
    count++;
  }

  /* Print out a description of the name tree in dot format */

  plotTreeHeader(Ta,"Symbol Tree - sorted by name");
  plotTreeNodes(Ta,symbolPrintLabel);
  plotTreeEdges(Ta);
  plotTreeFooter(Ta);

  /* Print out a description of the value tree in dot format */

  plotTreeHeader(Tb,"Symbol Tree - sorted by value");
  plotTreeNodes(Tb,symbolPrintLabel);
  plotTreeEdges(Tb);
  plotTreeFooter(Tb);

  /* Find some nodes */

  printf("/* finding min by value.");
  node = FindTreeNodeMin(Tb);
  if (node != NULL) {
    symP = (struct Symbol *)RetrieveTreeNodeElement(node);
    printf("  found it (%s at %lx).",symP->name,symP->value);
  } else {
    printf("  not found.");
  }
  printf("*/\n");

  printf("/* finding max by value.");
  node = FindTreeNodeMax(Tb);
  if (node != NULL) {
    symP = (struct Symbol *)RetrieveTreeNodeElement(node);
    printf("  found it (%s at %lx).",symP->name,symP->value);
  } else {
    printf("  not found.");
  }
  printf("*/\n");
    
  for (idx=0; findNames[idx] != NULL; idx++) {
    sym.name = findNames[idx];
    printf("/* finding %s.",findNames[idx]);
    node = FindTreeNode(Ta,symbolCompareByName,&sym);
    if (node != NULL) {
      symP = (struct Symbol *)RetrieveTreeNodeElement(node);
      printf("  found it (%s at %lx).",symP->name,symP->value);
    } else {
      printf("  not found.");
    }
    printf("*/\n");
  }

  /* Release all the memory associated with the trees. */

  DestroyTree(Ta);
  DestroyTree(Tb);

  /* Release all the memory associated with the Symbols. */

  freeSymbols(symTable);

  return 0;
}

/*---------------------------------------------------*/

/* Print a Symbol label in a format suitable for dot plots. */

void symbolPrintLabel(void *E) {
  if (E==NULL)
    printf("<NULL>");
  else
    printf("%s\\n%lx",((struct Symbol *)E)->name,((struct Symbol *)E)->value);
}

